@@ -1,12 +1,10 @@
-# Copyright (c) 2016 Claudiu Popa <pcmanticore@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
 """Astroid brain hints for some of the _io C objects."""
-
-import astroid
-
+from astroid.manager import AstroidManager
+from astroid.nodes import ClassDef
 
 BUFFERED = {"BufferedWriter", "BufferedReader"}
 TextIOWrapper = "TextIOWrapper"
@@ -15,9 +13,11 @@ BufferedWriter = "BufferedWriter"
 
 
 def _generic_io_transform(node, name, cls):
-    """Transform the given name, by adding the given *class* as a member of the node."""
+    """Transform the given name, by adding the given *class* as a member of the
+    node.
+    """
 
-    io_module = astroid.MANAGER.ast_from_module_name("_io")
+    io_module = AstroidManager().ast_from_module_name("_io")
     attribute_object = io_module[cls]
     instance = attribute_object.instantiate_class()
     node.locals[name] = [instance]
@@ -35,11 +35,9 @@ def _transform_buffered(node):
     return _generic_io_transform(node, name="raw", cls=FileIO)
 
 
-astroid.MANAGER.register_transform(
-    astroid.ClassDef, _transform_buffered, lambda node: node.name in BUFFERED
+AstroidManager().register_transform(
+    ClassDef, _transform_buffered, lambda node: node.name in BUFFERED
 )
-astroid.MANAGER.register_transform(
-    astroid.ClassDef,
-    _transform_text_io_wrapper,
-    lambda node: node.name == TextIOWrapper,
+AstroidManager().register_transform(
+    ClassDef, _transform_text_io_wrapper, lambda node: node.name == TextIOWrapper
 )
