@@ -1,43 +1,21 @@
-# Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
-# Copyright (c) 2013-2014 Google, Inc.
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
-# Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
-# Copyright (c) 2017 Michał Masłowski <m.maslowski@clearcode.cc>
-# Copyright (c) 2017 Calen Pennington <cale@edx.org>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2018 Daniel Martin <daniel.martin@crowdstrike.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2018 HoverHell <hoverhell@gmail.com>
-# Copyright (c) 2020 Leandro T. C. Melo <ltcmelo@gmail.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Kian Meng, Ang <kianmeng.ang@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
-# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""this module contains a set of functions to handle inference on astroid trees
-"""
+"""This module contains a set of functions to handle inference on astroid trees."""
+
+from __future__ import annotations
 
 import ast
 import functools
 import itertools
 import operator
-from typing import Any, Callable, Dict, Iterable, Optional
+import typing
+from collections.abc import Callable, Generator, Iterable, Iterator
+from typing import TYPE_CHECKING, Any, Optional, TypeVar, Union
 
-import wrapt
-
-from astroid import bases, decorators, helpers, nodes, protocols, util
+from astroid import bases, constraint, decorators, helpers, nodes, protocols, util
+from astroid.const import PY310_PLUS
 from astroid.context import (
     CallContext,
     InferenceContext,
@@ -49,6 +27,7 @@ from astroid.exceptions import (
     AstroidError,
     AstroidIndexError,
     AstroidTypeError,
+    AstroidValueError,
     AttributeInferenceError,
     InferenceError,
     NameInferenceError,
@@ -56,16 +35,42 @@ from astroid.exceptions import (
 )
 from astroid.interpreter import dunder_lookup
 from astroid.manager import AstroidManager
+from astroid.typing import (
+    InferenceErrorInfo,
+    InferenceResult,
+    SuccessfulInferenceResult,
+)
+
+if TYPE_CHECKING:
+    from astroid.objects import Property
 
 # Prevents circular imports
 objects = util.lazy_import("objects")
 
+_T = TypeVar("_T")
+_BaseContainerT = TypeVar("_BaseContainerT", bound=nodes.BaseContainer)
+_FunctionDefT = TypeVar("_FunctionDefT", bound=nodes.FunctionDef)
+
+GetFlowFactory = typing.Callable[
+    [
+        InferenceResult,
+        Optional[InferenceResult],
+        Union[nodes.AugAssign, nodes.BinOp],
+        InferenceResult,
+        Optional[InferenceResult],
+        InferenceContext,
+        InferenceContext,
+    ],
+    "list[functools.partial[Generator[InferenceResult, None, None]]]",
+]
 
 # .infer method ###############################################################
 
 
-def infer_end(self, context=None):
-    """Inference's end for nodes that yield themselves on inference
+def infer_end(
+    self: _T, context: InferenceContext | None = None, **kwargs: Any
+) -> Iterator[_T]:
+    """Inference's end for nodes that yield themselves on inference.
 
     These are objects for which inference does not have any semantic,
     such as Module or Consts.
@@ -73,17 +78,19 @@ def infer_end(self, context=None):
     yield self
 
 
-# We add ignores to all these assignments in this file
+# We add ignores to all assignments to methods
 # See https://github.com/python/mypy/issues/2427
-nodes.Module._infer = infer_end  # type: ignore[assignment]
-nodes.ClassDef._infer = infer_end  # type: ignore[assignment]
+nodes.Module._infer = infer_end
+nodes.ClassDef._infer = infer_end
 nodes.Lambda._infer = infer_end  # type: ignore[assignment]
 nodes.Const._infer = infer_end  # type: ignore[assignment]
 nodes.Slice._infer = infer_end  # type: ignore[assignment]
 
 
-def _infer_sequence_helper(node, context=None):
-    """Infer all values based on _BaseContainer.elts"""
+def _infer_sequence_helper(
+    node: _BaseContainerT, context: InferenceContext | None = None
+) -> list[SuccessfulInferenceResult]:
+    """Infer all values based on _BaseContainer.elts."""
     values = []
 
     for elt in node.elts:
@@ -105,7 +112,11 @@ def _infer_sequence_helper(node, context=None):
 
 
 @decorators.raise_if_nothing_inferred
-def infer_sequence(self, context=None):
+def infer_sequence(
+    self: _BaseContainerT,
+    context: InferenceContext | None = None,
+    **kwargs: Any,
+) -> Iterator[_BaseContainerT]:
     has_starred_named_expr = any(
         isinstance(e, (nodes.Starred, nodes.NamedExpr)) for e in self.elts
     )
@@ -126,7 +137,9 @@ nodes.Tuple._infer = infer_sequence  # type: ignore[assignment]
 nodes.Set._infer = infer_sequence  # type: ignore[assignment]
 
 
-def infer_map(self, context=None):
+def infer_map(
+    self: nodes.Dict, context: InferenceContext | None = None
+) -> Iterator[nodes.Dict]:
     if not any(isinstance(k, nodes.DictUnpack) for k, _ in self.items):
         yield self
     else:
@@ -136,8 +149,11 @@ def infer_map(self, context=None):
         yield new_seq
 
 
-def _update_with_replacement(lhs_dict, rhs_dict):
-    """Delete nodes that equate to duplicate keys
+def _update_with_replacement(
+    lhs_dict: dict[SuccessfulInferenceResult, SuccessfulInferenceResult],
+    rhs_dict: dict[SuccessfulInferenceResult, SuccessfulInferenceResult],
+) -> dict[SuccessfulInferenceResult, SuccessfulInferenceResult]:
+    """Delete nodes that equate to duplicate keys.
 
     Since an astroid node doesn't 'equal' another node with the same value,
     this function uses the as_string method to make sure duplicate keys
@@ -145,12 +161,12 @@ def _update_with_replacement(lhs_dict, rhs_dict):
 
     Note that both the key and the value are astroid nodes
 
-    Fixes issue with DictUnpack causing duplicte keys
+    Fixes issue with DictUnpack causing duplicate keys
     in inferred Dict items
 
-    :param dict(nodes.NodeNG, nodes.NodeNG) lhs_dict: Dictionary to 'merge' nodes into
-    :param dict(nodes.NodeNG, nodes.NodeNG) rhs_dict: Dictionary with nodes to pull from
-    :return dict(nodes.NodeNG, nodes.NodeNG): merged dictionary of nodes
+    :param lhs_dict: Dictionary to 'merge' nodes into
+    :param rhs_dict: Dictionary with nodes to pull from
+    :return : merged dictionary of nodes
     """
     combined_dict = itertools.chain(lhs_dict.items(), rhs_dict.items())
     # Overwrite keys which have the same string values
@@ -159,9 +175,11 @@ def _update_with_replacement(lhs_dict, rhs_dict):
     return dict(string_map.values())
 
 
-def _infer_map(node, context):
-    """Infer all values based on Dict.items"""
-    values = {}
+def _infer_map(
+    node: nodes.Dict, context: InferenceContext | None
+) -> dict[SuccessfulInferenceResult, SuccessfulInferenceResult]:
+    """Infer all values based on Dict.items."""
+    values: dict[SuccessfulInferenceResult, SuccessfulInferenceResult] = {}
     for name, value in node.items:
         if isinstance(name, nodes.DictUnpack):
             double_starred = helpers.safe_infer(value, context)
@@ -173,17 +191,18 @@ def _infer_map(node, context):
             values = _update_with_replacement(values, unpack_items)
         else:
             key = helpers.safe_infer(name, context=context)
-            value = helpers.safe_infer(value, context=context)
-            if any(not elem for elem in (key, value)):
+            safe_value = helpers.safe_infer(value, context=context)
+            if any(not elem for elem in (key, safe_value)):
                 raise InferenceError(node=node, context=context)
-            values = _update_with_replacement(values, {key: value})
+            # safe_value is SuccessfulInferenceResult as bool(Uninferable) == False
+            values = _update_with_replacement(values, {key: safe_value})
     return values
 
 
 nodes.Dict._infer = infer_map  # type: ignore[assignment]
 
 
-def _higher_function_scope(node):
+def _higher_function_scope(node: nodes.NodeNG) -> nodes.FunctionDef | None:
     """Search for the first function which encloses the given
     scope. This can be used for looking up in that function's
     scope, in case looking up in a lower scope for a particular
@@ -199,12 +218,16 @@ def _higher_function_scope(node):
     while current.parent and not isinstance(current.parent, nodes.FunctionDef):
         current = current.parent
     if current and current.parent:
-        return current.parent
+        return current.parent  # type: ignore[no-any-return]
     return None
 
 
-def infer_name(self, context=None):
-    """infer a Name: use name lookup rules"""
+def infer_name(
+    self: nodes.Name | nodes.AssignName,
+    context: InferenceContext | None = None,
+    **kwargs: Any,
+) -> Generator[InferenceResult, None, None]:
+    """Infer a Name: use name lookup rules."""
     frame, stmts = self.lookup(self.name)
     if not stmts:
         # Try to see if the name is enclosed in a nested function
@@ -219,10 +242,14 @@ def infer_name(self, context=None):
             )
     context = copy_context(context)
     context.lookupname = self.name
+    context.constraints[self.name] = constraint.get_constraints(self, frame)
+
     return bases._infer_stmts(stmts, context, frame)
 
 
 # pylint: disable=no-value-for-parameter
+# The order of the decorators here is important
+# See https://github.com/PyCQA/astroid/commit/0a8a75db30da060a24922e05048bc270230f5
 nodes.Name._infer = decorators.raise_if_nothing_inferred(
     decorators.path_wrapper(infer_name)
 )
@@ -231,8 +258,10 @@ nodes.AssignName.infer_lhs = infer_name  # won't work with a path wrapper
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_call(self, context=None):
-    """infer a Call node by trying to guess what the function returns"""
+def infer_call(
+    self: nodes.Call, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, InferenceErrorInfo]:
+    """Infer a Call node by trying to guess what the function returns."""
     callcontext = copy_context(context)
     callcontext.boundnode = None
     if context is not None:
@@ -250,7 +279,7 @@ def infer_call(self, context=None):
                 yield from callee.infer_call_result(caller=self, context=callcontext)
         except InferenceError:
             continue
-    return dict(node=self, context=context)
+    return InferenceErrorInfo(node=self, context=context)
 
 
 nodes.Call._infer = infer_call  # type: ignore[assignment]
@@ -258,8 +287,14 @@ nodes.Call._infer = infer_call  # type: ignore[assignment]
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_import(self, context=None, asname=True):
-    """infer an Import node: return the imported module/object"""
+def infer_import(
+    self: nodes.Import,
+    context: InferenceContext | None = None,
+    asname: bool = True,
+    **kwargs: Any,
+) -> Generator[nodes.Module, None, None]:
+    """Infer an Import node: return the imported module/object."""
+    context = context or InferenceContext()
     name = context.lookupname
     if name is None:
         raise InferenceError(node=self, context=context)
@@ -278,8 +313,14 @@ nodes.Import._infer = infer_import
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_import_from(self, context=None, asname=True):
-    """infer a ImportFrom node: return the imported module/object"""
+def infer_import_from(
+    self: nodes.ImportFrom,
+    context: InferenceContext | None = None,
+    asname: bool = True,
+    **kwargs: Any,
+) -> Generator[InferenceResult, None, None]:
+    """Infer a ImportFrom node: return the imported module/object."""
+    context = context or InferenceContext()
     name = context.lookupname
     if name is None:
         raise InferenceError(node=self, context=context)
@@ -308,19 +349,26 @@ def infer_import_from(self, context=None, asname=True):
 nodes.ImportFrom._infer = infer_import_from  # type: ignore[assignment]
 
 
-def infer_attribute(self, context=None):
-    """infer an Attribute node by using getattr on the associated object"""
+def infer_attribute(
+    self: nodes.Attribute | nodes.AssignAttr,
+    context: InferenceContext | None = None,
+    **kwargs: Any,
+) -> Generator[InferenceResult, None, InferenceErrorInfo]:
+    """Infer an Attribute node by using getattr on the associated object."""
     for owner in self.expr.infer(context):
         if owner is util.Uninferable:
             yield owner
             continue
 
-        if not context:
-            context = InferenceContext()
-
+        context = copy_context(context)
         old_boundnode = context.boundnode
         try:
             context.boundnode = owner
+            if isinstance(owner, (nodes.ClassDef, bases.Instance)):
+                frame = owner if isinstance(owner, nodes.ClassDef) else owner._proxied
+                context.constraints[self.attrname] = constraint.get_constraints(
+                    self, frame=frame
+                )
             yield from owner.igetattr(self.attrname, context)
         except (
             AttributeInferenceError,
@@ -330,9 +378,11 @@ def infer_attribute(self, context=None):
             pass
         finally:
             context.boundnode = old_boundnode
-    return dict(node=self, context=context)
+    return InferenceErrorInfo(node=self, context=context)
 
 
+# The order of the decorators here is important
+# See https://github.com/PyCQA/astroid/commit/0a8a75db30da060a24922e05048bc270230f5
 nodes.Attribute._infer = decorators.raise_if_nothing_inferred(
     decorators.path_wrapper(infer_attribute)
 )
@@ -342,8 +392,10 @@ nodes.AssignAttr.infer_lhs = decorators.raise_if_nothing_inferred(infer_attribut
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_global(self, context=None):
-    if context.lookupname is None:
+def infer_global(
+    self: nodes.Global, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
+    if context is None or context.lookupname is None:
         raise InferenceError(node=self, context=context)
     try:
         return bases._infer_stmts(self.root().getattr(context.lookupname), context)
@@ -359,8 +411,10 @@ nodes.Global._infer = infer_global  # type: ignore[assignment]
 _SUBSCRIPT_SENTINEL = object()
 
 
-def infer_subscript(self, context=None):
-    """Inference for subscripts
+def infer_subscript(
+    self: nodes.Subscript, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, InferenceErrorInfo | None]:
+    """Inference for subscripts.
 
     We're understanding if the index is a Const
     or a slice, passing the result of inference
@@ -397,6 +451,7 @@ def infer_subscript(self, context=None):
             except (
                 AstroidTypeError,
                 AstroidIndexError,
+                AstroidValueError,
                 AttributeInferenceError,
                 AttributeError,
             ) as exc:
@@ -411,10 +466,12 @@ def infer_subscript(self, context=None):
             found_one = True
 
     if found_one:
-        return dict(node=self, context=context)
+        return InferenceErrorInfo(node=self, context=context)
     return None
 
 
+# The order of the decorators here is important
+# See https://github.com/PyCQA/astroid/commit/0a8a75db30da060a24922e05048bc270230f5
 nodes.Subscript._infer = decorators.raise_if_nothing_inferred(  # type: ignore[assignment]
     decorators.path_wrapper(infer_subscript)
 )
@@ -423,7 +480,9 @@ nodes.Subscript.infer_lhs = decorators.raise_if_nothing_inferred(infer_subscript
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def _infer_boolop(self, context=None):
+def _infer_boolop(
+    self: nodes.BoolOp, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, InferenceErrorInfo | None]:
     """Infer a boolean operation (and / or / not).
 
     The function will calculate the boolean operation
@@ -437,12 +496,12 @@ def _infer_boolop(self, context=None):
         predicate = operator.not_
 
     try:
-        values = [value.infer(context=context) for value in values]
+        inferred_values = [value.infer(context=context) for value in values]
     except InferenceError:
         yield util.Uninferable
         return None
 
-    for pair in itertools.product(*values):
+    for pair in itertools.product(*inferred_values):
         if any(item is util.Uninferable for item in pair):
             # Can't infer the final result, just yield Uninferable.
             yield util.Uninferable
@@ -471,7 +530,7 @@ def _infer_boolop(self, context=None):
         else:
             yield value
 
-    return dict(node=self, context=context)
+    return InferenceErrorInfo(node=self, context=context)
 
 
 nodes.BoolOp._infer = _infer_boolop
@@ -480,7 +539,15 @@ nodes.BoolOp._infer = _infer_boolop
 # UnaryOp, BinOp and AugAssign inferences
 
 
-def _filter_operation_errors(self, infer_callable, context, error):
+def _filter_operation_errors(
+    self: _T,
+    infer_callable: Callable[
+        [_T, InferenceContext | None],
+        Generator[InferenceResult | util.BadOperationMessage, None, None],
+    ],
+    context: InferenceContext | None,
+    error: type[util.BadOperationMessage],
+) -> Generator[InferenceResult, None, None]:
     for result in infer_callable(self, context):
         if isinstance(result, error):
             # For the sake of .infer(), we don't care about operation
@@ -491,7 +558,9 @@ def _filter_operation_errors(self, infer_callable, context, error):
             yield result
 
 
-def _infer_unaryop(self, context=None):
+def _infer_unaryop(
+    self: nodes.UnaryOp, context: InferenceContext | None = None
+) -> Generator[InferenceResult | util.BadUnaryOperationMessage, None, None]:
     """Infer what an UnaryOp should return when evaluated."""
     for operand in self.operand.infer(context):
         try:
@@ -540,48 +609,113 @@ def _infer_unaryop(self, context=None):
                         yield operand
                     else:
                         yield result
-                except AttributeInferenceError as exc:
+                except AttributeInferenceError as inner_exc:
                     # The unary operation special method was not found.
-                    yield util.BadUnaryOperationMessage(operand, self.op, exc)
+                    yield util.BadUnaryOperationMessage(operand, self.op, inner_exc)
                 except InferenceError:
                     yield util.Uninferable
 
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_unaryop(self, context=None):
+def infer_unaryop(
+    self: nodes.UnaryOp, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, InferenceErrorInfo]:
     """Infer what an UnaryOp should return when evaluated."""
     yield from _filter_operation_errors(
         self, _infer_unaryop, context, util.BadUnaryOperationMessage
     )
-    return dict(node=self, context=context)
+    return InferenceErrorInfo(node=self, context=context)
 
 
 nodes.UnaryOp._infer_unaryop = _infer_unaryop
 nodes.UnaryOp._infer = infer_unaryop
 
 
-def _is_not_implemented(const):
+def _is_not_implemented(const) -> bool:
     """Check if the given const node is NotImplemented."""
     return isinstance(const, nodes.Const) and const.value is NotImplemented
 
 
-def _invoke_binop_inference(instance, opnode, op, other, context, method_name):
+def _infer_old_style_string_formatting(
+    instance: nodes.Const, other: nodes.NodeNG, context: InferenceContext
+) -> tuple[type[util.Uninferable] | nodes.Const]:
+    """Infer the result of '"string" % ...'.
+
+    TODO: Instead of returning Uninferable we should rely
+    on the call to '%' to see if the result is actually uninferable.
+    """
+    if isinstance(other, nodes.Tuple):
+        if util.Uninferable in other.elts:
+            return (util.Uninferable,)
+        inferred_positional = [helpers.safe_infer(i, context) for i in other.elts]
+        if all(isinstance(i, nodes.Const) for i in inferred_positional):
+            values = tuple(i.value for i in inferred_positional)
+        else:
+            values = None
+    elif isinstance(other, nodes.Dict):
+        values: dict[Any, Any] = {}
+        for pair in other.items:
+            key = helpers.safe_infer(pair[0], context)
+            if not isinstance(key, nodes.Const):
+                return (util.Uninferable,)
+            value = helpers.safe_infer(pair[1], context)
+            if not isinstance(value, nodes.Const):
+                return (util.Uninferable,)
+            values[key.value] = value.value
+    elif isinstance(other, nodes.Const):
+        values = other.value
+    else:
+        return (util.Uninferable,)
+
+    try:
+        return (nodes.const_factory(instance.value % values),)
+    except (TypeError, KeyError, ValueError):
+        return (util.Uninferable,)
+
+
+def _invoke_binop_inference(
+    instance: InferenceResult,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    op: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    method_name: str,
+) -> Generator[InferenceResult, None, None]:
     """Invoke binary operation inference on the given instance."""
     methods = dunder_lookup.lookup(instance, method_name)
     context = bind_context_to_node(context, instance)
     method = methods[0]
     context.callcontext.callee = method
+
+    if (
+        isinstance(instance, nodes.Const)
+        and isinstance(instance.value, str)
+        and op == "%"
+    ):
+        return iter(_infer_old_style_string_formatting(instance, other, context))
+
     try:
         inferred = next(method.infer(context=context))
     except StopIteration as e:
         raise InferenceError(node=method, context=context) from e
     if inferred is util.Uninferable:
         raise InferenceError
+    if not isinstance(
+        instance, (nodes.Const, nodes.Tuple, nodes.List, nodes.ClassDef, bases.Instance)
+    ):
+        raise InferenceError  # pragma: no cover # Used as a failsafe
     return instance.infer_binary_op(opnode, op, other, context, inferred)
 
 
-def _aug_op(instance, opnode, op, other, context, reverse=False):
+def _aug_op(
+    instance: InferenceResult,
+    opnode: nodes.AugAssign,
+    op: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    reverse: bool = False,
+) -> functools.partial[Generator[InferenceResult, None, None]]:
     """Get an inference callable for an augmented binary operation."""
     method_name = protocols.AUGMENTED_OP_METHOD[op]
     return functools.partial(
@@ -595,7 +729,14 @@ def _aug_op(instance, opnode, op, other, context, reverse=False):
     )
 
 
-def _bin_op(instance, opnode, op, other, context, reverse=False):
+def _bin_op(
+    instance: InferenceResult,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    op: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    reverse: bool = False,
+) -> functools.partial[Generator[InferenceResult, None, None]]:
     """Get an inference callable for a normal binary operation.
 
     If *reverse* is True, then the reflected method will be used instead.
@@ -615,6 +756,14 @@ def _bin_op(instance, opnode, op, other, context, reverse=False):
     )
 
 
+def _bin_op_or_union_type(
+    left: bases.UnionType | nodes.ClassDef | nodes.Const,
+    right: bases.UnionType | nodes.ClassDef | nodes.Const,
+) -> Generator[InferenceResult, None, None]:
+    """Create a new UnionType instance for binary or, e.g. int | str."""
+    yield bases.UnionType(left, right)
+
+
 def _get_binop_contexts(context, left, right):
     """Get contexts for binary operations.
 
@@ -631,14 +780,20 @@ def _get_binop_contexts(context, left, right):
         yield new_context
 
 
-def _same_type(type1, type2):
+def _same_type(type1, type2) -> bool:
     """Check if type1 is the same as type2."""
     return type1.qname() == type2.qname()
 
 
 def _get_binop_flow(
-    left, left_type, binary_opnode, right, right_type, context, reverse_context
-):
+    left: InferenceResult,
+    left_type: InferenceResult | None,
+    binary_opnode: nodes.AugAssign | nodes.BinOp,
+    right: InferenceResult,
+    right_type: InferenceResult | None,
+    context: InferenceContext,
+    reverse_context: InferenceContext,
+) -> list[functools.partial[Generator[InferenceResult, None, None]]]:
     """Get the flow for binary operations.
 
     The rules are a bit messy:
@@ -668,12 +823,34 @@ def _get_binop_flow(
             _bin_op(left, binary_opnode, op, right, context),
             _bin_op(right, binary_opnode, op, left, reverse_context, reverse=True),
         ]
+
+    if (
+        PY310_PLUS
+        and op == "|"
+        and (
+            isinstance(left, (bases.UnionType, nodes.ClassDef))
+            or isinstance(left, nodes.Const)
+            and left.value is None
+        )
+        and (
+            isinstance(right, (bases.UnionType, nodes.ClassDef))
+            or isinstance(right, nodes.Const)
+            and right.value is None
+        )
+    ):
+        methods.extend([functools.partial(_bin_op_or_union_type, left, right)])
     return methods
 
 
 def _get_aug_flow(
-    left, left_type, aug_opnode, right, right_type, context, reverse_context
-):
+    left: InferenceResult,
+    left_type: InferenceResult | None,
+    aug_opnode: nodes.AugAssign,
+    right: InferenceResult,
+    right_type: InferenceResult | None,
+    context: InferenceContext,
+    reverse_context: InferenceContext,
+) -> list[functools.partial[Generator[InferenceResult, None, None]]]:
     """Get the flow for augmented binary operations.
 
     The rules are a bit messy:
@@ -716,8 +893,14 @@ def _get_aug_flow(
     return methods
 
 
-def _infer_binary_operation(left, right, binary_opnode, context, flow_factory):
-    """Infer a binary operation between a left operand and a right operand
+def _infer_binary_operation(
+    left: InferenceResult,
+    right: InferenceResult,
+    binary_opnode: nodes.AugAssign | nodes.BinOp,
+    context: InferenceContext,
+    flow_factory: GetFlowFactory,
+) -> Generator[InferenceResult | util.BadBinaryOperationMessage, None, None]:
+    """Infer a binary operation between a left operand and a right operand.
 
     This is used by both normal binary operations and augmented binary
     operations, the only difference is the flow factory used.
@@ -760,7 +943,9 @@ def _infer_binary_operation(left, right, binary_opnode, context, flow_factory):
     yield util.BadBinaryOperationMessage(left_type, binary_opnode.op, right_type)
 
 
-def _infer_binop(self, context):
+def _infer_binop(
+    self: nodes.BinOp, context: InferenceContext | None = None
+) -> Generator[InferenceResult | util.BadBinaryOperationMessage, None, None]:
     """Binary operation inference logic."""
     left = self.left
     right = self.right
@@ -787,7 +972,9 @@ def _infer_binop(self, context):
 
 @decorators.yes_if_nothing_inferred
 @decorators.path_wrapper
-def infer_binop(self, context=None):
+def infer_binop(
+    self: nodes.BinOp, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
     return _filter_operation_errors(
         self, _infer_binop, context, util.BadBinaryOperationMessage
     )
@@ -796,7 +983,7 @@ def infer_binop(self, context=None):
 nodes.BinOp._infer_binop = _infer_binop
 nodes.BinOp._infer = infer_binop
 
-COMPARE_OPS: Dict[str, Callable[[Any, Any], bool]] = {
+COMPARE_OPS: dict[str, Callable[[Any, Any], bool]] = {
     "==": operator.eq,
     "!=": operator.ne,
     "<": operator.lt,
@@ -821,7 +1008,7 @@ def _to_literal(node: nodes.NodeNG) -> Any:
 
 def _do_compare(
     left_iter: Iterable[nodes.NodeNG], op: str, right_iter: Iterable[nodes.NodeNG]
-) -> "bool | type[util.Uninferable]":
+) -> bool | type[util.Uninferable]:
     """
     If all possible combinations are either True or False, return that:
     >>> _do_compare([1, 2], '<=', [3, 4])
@@ -834,7 +1021,7 @@ def _do_compare(
     >>> _do_compare([1, 3], '<=', [2, 4])
     util.Uninferable
     """
-    retval = None
+    retval: bool | None = None
     if op in UNINFERABLE_OPS:
         return util.Uninferable
     op_func = COMPARE_OPS[op]
@@ -859,14 +1046,15 @@ def _do_compare(
             return util.Uninferable
             # (or both, but "True | False" is basically the same)
 
+    assert retval is not None
     return retval  # it was all the same value
 
 
 def _infer_compare(
-    self: nodes.Compare, context: Optional[InferenceContext] = None
-) -> Any:
+    self: nodes.Compare, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[nodes.Const | type[util.Uninferable], None, None]:
     """Chained comparison inference logic."""
-    retval = True
+    retval: bool | type[util.Uninferable] = True
 
     ops = self.ops
     left_node = self.left
@@ -884,7 +1072,7 @@ def _infer_compare(
             break  # short-circuit
         lhs = rhs  # continue
     if retval is util.Uninferable:
-        yield retval
+        yield retval  # type: ignore[misc]
     else:
         yield nodes.Const(retval)
 
@@ -892,10 +1080,11 @@ def _infer_compare(
 nodes.Compare._infer = _infer_compare  # type: ignore[assignment]
 
 
-def _infer_augassign(self, context=None):
+def _infer_augassign(
+    self: nodes.AugAssign, context: InferenceContext | None = None
+) -> Generator[InferenceResult | util.BadBinaryOperationMessage, None, None]:
     """Inference logic for augmented binary operations."""
-    if context is None:
-        context = InferenceContext()
+    context = context or InferenceContext()
 
     rhs_context = context.clone()
 
@@ -921,7 +1110,9 @@ def _infer_augassign(self, context=None):
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_augassign(self, context=None):
+def infer_augassign(
+    self: nodes.AugAssign, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
     return _filter_operation_errors(
         self, _infer_augassign, context, util.BadBinaryOperationMessage
     )
@@ -934,11 +1125,12 @@ nodes.AugAssign._infer = infer_augassign
 
 
 @decorators.raise_if_nothing_inferred
-def infer_arguments(self, context=None):
-    name = context.lookupname
-    if name is None:
+def infer_arguments(
+    self: nodes.Arguments, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
+    if context is None or context.lookupname is None:
         raise InferenceError(node=self, context=context)
-    return protocols._arguments_infer_argname(self, name, context)
+    return protocols._arguments_infer_argname(self, context.lookupname, context)
 
 
 nodes.Arguments._infer = infer_arguments  # type: ignore[assignment]
@@ -946,9 +1138,13 @@ nodes.Arguments._infer = infer_arguments  # type: ignore[assignment]
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_assign(self, context=None):
-    """infer a AssignName/AssignAttr: need to inspect the RHS part of the
-    assign node
+def infer_assign(
+    self: nodes.AssignName | nodes.AssignAttr,
+    context: InferenceContext | None = None,
+    **kwargs: Any,
+) -> Generator[InferenceResult, None, None]:
+    """Infer a AssignName/AssignAttr: need to inspect the RHS part of the
+    assign node.
     """
     if isinstance(self.parent, nodes.AugAssign):
         return self.parent.infer(context)
@@ -963,7 +1159,9 @@ nodes.AssignAttr._infer = infer_assign
 
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
-def infer_empty_node(self, context=None):
+def infer_empty_node(
+    self: nodes.EmptyNode, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
     if not self.has_underlying_object():
         yield util.Uninferable
     else:
@@ -978,18 +1176,10 @@ def infer_empty_node(self, context=None):
 nodes.EmptyNode._infer = infer_empty_node  # type: ignore[assignment]
 
 
-@decorators.raise_if_nothing_inferred
-def infer_index(self, context=None):
-    return self.value.infer(context)
-
-
-nodes.Index._infer = infer_index  # type: ignore[assignment]
-
-
-def _populate_context_lookup(call, context):
+def _populate_context_lookup(call: nodes.Call, context: InferenceContext | None):
     # Allows context to be saved for later
     # for inference inside a function
-    context_lookup = {}
+    context_lookup: dict[InferenceResult, InferenceContext] = {}
     if context is None:
         return context_lookup
     for arg in call.args:
@@ -1004,8 +1194,10 @@ def _populate_context_lookup(call, context):
 
 
 @decorators.raise_if_nothing_inferred
-def infer_ifexp(self, context=None):
-    """Support IfExp inference
+def infer_ifexp(
+    self: nodes.IfExp, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[InferenceResult, None, None]:
+    """Support IfExp inference.
 
     If we can't infer the truthiness of the condition, we default
     to inferring both branches. Otherwise, we infer either branch
@@ -1039,42 +1231,38 @@ def infer_ifexp(self, context=None):
 nodes.IfExp._infer = infer_ifexp  # type: ignore[assignment]
 
 
-# pylint: disable=dangerous-default-value
-@wrapt.decorator
-def _cached_generator(func, instance, args, kwargs, _cache={}):  # noqa: B006
-    node = args[0]
-    try:
-        return iter(_cache[func, id(node)])
-    except KeyError:
-        result = func(*args, **kwargs)
-        # Need to keep an iterator around
-        original, copy = itertools.tee(result)
-        _cache[func, id(node)] = list(copy)
-        return original
-
-
-# When inferring a property, we instantiate a new `objects.Property` object,
-# which in turn, because it inherits from `FunctionDef`, sets itself in the locals
-# of the wrapping frame. This means that every time we infer a property, the locals
-# are mutated with a new instance of the property. This is why we cache the result
-# of the function's inference.
-@_cached_generator
-def infer_functiondef(self, context=None):
+def infer_functiondef(
+    self: _FunctionDefT, context: InferenceContext | None = None, **kwargs: Any
+) -> Generator[Property | _FunctionDefT, None, InferenceErrorInfo]:
     if not self.decorators or not bases._is_property(self):
         yield self
-        return dict(node=self, context=context)
+        return InferenceErrorInfo(node=self, context=context)
+
+    # When inferring a property, we instantiate a new `objects.Property` object,
+    # which in turn, because it inherits from `FunctionDef`, sets itself in the locals
+    # of the wrapping frame. This means that every time we infer a property, the locals
+    # are mutated with a new instance of the property. To avoid this, we detect this
+    # scenario and avoid passing the `parent` argument to the constructor.
+    parent_frame = self.parent.frame(future=True)
+    property_already_in_parent_locals = self.name in parent_frame.locals and any(
+        isinstance(val, objects.Property) for val in parent_frame.locals[self.name]
+    )
+    # We also don't want to pass parent if the definition is within a Try node
+    if isinstance(self.parent, (nodes.TryExcept, nodes.TryFinally, nodes.If)):
+        property_already_in_parent_locals = True
 
     prop_func = objects.Property(
         function=self,
         name=self.name,
-        doc=self.doc,
         lineno=self.lineno,
-        parent=self.parent,
+        parent=self.parent if not property_already_in_parent_locals else None,
         col_offset=self.col_offset,
     )
-    prop_func.postinit(body=[], args=self.args)
+    if property_already_in_parent_locals:
+        prop_func.parent = self.parent
+    prop_func.postinit(body=[], args=self.args, doc_node=self.doc_node)
     yield prop_func
-    return dict(node=self, context=context)
+    return InferenceErrorInfo(node=self, context=context)
 
 
-nodes.FunctionDef._infer = infer_functiondef  # type: ignore[assignment]
+nodes.FunctionDef._infer = infer_functiondef
