@@ -1,40 +1,18 @@
-# Copyright (c) 2009-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Google, Inc.
-# Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
-# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2017-2018 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
-# Copyright (c) 2017 rr- <rr-@sakuya.pl>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 HoverHell <hoverhell@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Vilnis Termanis <vilnis.termanis@iotics.com>
-# Copyright (c) 2020 Ram Rachum <ram@rachum.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Tushar Sadhwani <86737547+tushar-deepsource@users.noreply.github.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 doranid <ddandd@gmail.com>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""this module contains a set of functions to handle python protocols for nodes
+"""This module contains a set of functions to handle python protocols for nodes
 where it makes sense.
 """
 
+from __future__ import annotations
+
 import collections
 import itertools
 import operator as operator_mod
-import sys
-from typing import Any, Generator, List, Optional, Union
+from collections.abc import Callable, Generator, Iterator, Sequence
+from typing import Any, TypeVar
 
 from astroid import arguments, bases, decorators, helpers, nodes, util
 from astroid.const import Context
@@ -47,21 +25,24 @@ from astroid.exceptions import (
     NoDefault,
 )
 from astroid.nodes import node_classes
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from astroid.typing import (
+    ConstFactoryResult,
+    InferenceResult,
+    SuccessfulInferenceResult,
+)
 
 raw_building = util.lazy_import("raw_building")
 objects = util.lazy_import("objects")
 
 
-def _reflected_name(name):
+_TupleListNodeT = TypeVar("_TupleListNodeT", nodes.Tuple, nodes.List)
+
+
+def _reflected_name(name) -> str:
     return "__r" + name[2:]
 
 
-def _augmented_name(name):
+def _augmented_name(name) -> str:
     return "__i" + name[2:]
 
 
@@ -95,7 +76,7 @@ UNARY_OP_METHOD = {
     "~": "__invert__",
     "not": None,  # XXX not '__nonzero__'
 }
-_UNARY_OPERATORS = {
+_UNARY_OPERATORS: dict[str, Callable[[Any], Any]] = {
     "+": operator_mod.pos,
     "-": operator_mod.neg,
     "~": operator_mod.invert,
@@ -103,9 +84,16 @@ _UNARY_OPERATORS = {
 }
 
 
-def _infer_unary_op(obj, op):
-    func = _UNARY_OPERATORS[op]
-    value = func(obj)
+def _infer_unary_op(obj: Any, op: str) -> ConstFactoryResult:
+    """Perform unary operation on `obj`, unless it is `NotImplemented`.
+
+    Can raise TypeError if operation is unsupported.
+    """
+    if obj is NotImplemented:
+        value = obj
+    else:
+        func = _UNARY_OPERATORS[op]
+        value = func(obj)
     return nodes.const_factory(value)
 
 
@@ -123,7 +111,7 @@ BIN_OP_IMPL = {
     "/": lambda a, b: a / b,
     "//": lambda a, b: a // b,
     "*": lambda a, b: a * b,
-    "**": lambda a, b: a ** b,
+    "**": lambda a, b: a**b,
     "%": lambda a, b: a % b,
     "&": lambda a, b: a & b,
     "|": lambda a, b: a | b,
@@ -137,9 +125,24 @@ for _KEY, _IMPL in list(BIN_OP_IMPL.items()):
 
 
 @decorators.yes_if_nothing_inferred
-def const_infer_binary_op(self, opnode, operator, other, context, _):
+def const_infer_binary_op(
+    self: nodes.Const,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    operator: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    _: SuccessfulInferenceResult,
+) -> Generator[ConstFactoryResult | type[util.Uninferable], None, None]:
     not_implemented = nodes.Const(NotImplemented)
     if isinstance(other, nodes.Const):
+        if (
+            operator == "**"
+            and isinstance(self.value, (int, float))
+            and isinstance(other.value, (int, float))
+            and (self.value > 1e5 or other.value > 1e5)
+        ):
+            yield not_implemented
+            return
         try:
             impl = BIN_OP_IMPL[operator]
             try:
@@ -161,7 +164,12 @@ def const_infer_binary_op(self, opnode, operator, other, context, _):
 nodes.Const.infer_binary_op = const_infer_binary_op
 
 
-def _multiply_seq_by_int(self, opnode, other, context):
+def _multiply_seq_by_int(
+    self: _TupleListNodeT,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    other: nodes.Const,
+    context: InferenceContext,
+) -> _TupleListNodeT:
     node = self.__class__(parent=opnode)
     filtered_elts = (
         helpers.safe_infer(elt, context) or util.Uninferable
@@ -172,7 +180,9 @@ def _multiply_seq_by_int(self, opnode, other, context):
     return node
 
 
-def _filter_uninferable_nodes(elts, context):
+def _filter_uninferable_nodes(
+    elts: Sequence[InferenceResult], context: InferenceContext
+) -> Iterator[SuccessfulInferenceResult]:
     for elt in elts:
         if elt is util.Uninferable:
             yield nodes.Unknown()
@@ -185,7 +195,22 @@ def _filter_uninferable_nodes(elts, context):
 
 
 @decorators.yes_if_nothing_inferred
-def tl_infer_binary_op(self, opnode, operator, other, context, method):
+def tl_infer_binary_op(
+    self: _TupleListNodeT,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    operator: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    method: SuccessfulInferenceResult,
+) -> Generator[_TupleListNodeT | nodes.Const | type[util.Uninferable], None, None]:
+    """Infer a binary operation on a tuple or list.
+
+    The instance on which the binary operation is performed is a tuple
+    or list. This refers to the left-hand side of the operation, so:
+    'tuple() + 1' or '[] + A()'
+    """
+    # For tuples and list the boundnode is no longer the tuple or list instance
+    context.boundnode = None
     not_implemented = nodes.Const(NotImplemented)
     if isinstance(other, self.__class__) and operator == "+":
         node = self.__class__(parent=opnode)
@@ -217,7 +242,14 @@ nodes.List.infer_binary_op = tl_infer_binary_op
 
 
 @decorators.yes_if_nothing_inferred
-def instance_class_infer_binary_op(self, opnode, operator, other, context, method):
+def instance_class_infer_binary_op(
+    self: bases.Instance | nodes.ClassDef,
+    opnode: nodes.AugAssign | nodes.BinOp,
+    operator: str,
+    other: InferenceResult,
+    context: InferenceContext,
+    method: SuccessfulInferenceResult,
+) -> Generator[InferenceResult, None, None]:
     return method.infer_call_result(self, context)
 
 
@@ -227,7 +259,7 @@ nodes.ClassDef.infer_binary_op = instance_class_infer_binary_op
 
 # assignment ##################################################################
 
-"""the assigned_stmts method is responsible to return the assigned statement
+"""The assigned_stmts method is responsible to return the assigned statement
 (e.g. not inferred) according to the assignment type.
 
 The `assign_path` argument is used to record the lhs path of the original node.
@@ -240,7 +272,7 @@ to any intermediary inference necessary.
 
 
 def _resolve_looppart(parts, assign_path, context):
-    """recursive function to resolve multiple assignments on loops"""
+    """Recursive function to resolve multiple assignments on loops."""
     assign_path = assign_path[:]
     index = assign_path.pop(0)
     for part in parts:
@@ -252,6 +284,11 @@ def _resolve_looppart(parts, assign_path, context):
             itered = part.itered()
         except TypeError:
             continue
+        try:
+            if isinstance(itered[index], (nodes.Const, nodes.Name)):
+                itered = [part]
+        except IndexError:
+            pass
         for stmt in itered:
             index_node = nodes.Const(index)
             try:
@@ -277,21 +314,31 @@ def _resolve_looppart(parts, assign_path, context):
 
 @decorators.raise_if_nothing_inferred
 def for_assigned_stmts(
-    self: Union[nodes.For, nodes.Comprehension],
+    self: nodes.For | nodes.Comprehension,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     if isinstance(self, nodes.AsyncFor) or getattr(self, "is_async", False):
         # Skip inferring of async code for now
-        return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+        return {
+            "node": self,
+            "unknown": node,
+            "assign_path": assign_path,
+            "context": context,
+        }
     if assign_path is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
                 yield from lst.elts
     else:
         yield from _resolve_looppart(self.iter.infer(context), assign_path, context)
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.For.assigned_stmts = for_assigned_stmts
@@ -299,15 +346,15 @@ nodes.Comprehension.assigned_stmts = for_assigned_stmts
 
 
 def sequence_assigned_stmts(
-    self: Union[nodes.Tuple, nodes.List],
+    self: nodes.Tuple | nodes.List,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     if assign_path is None:
         assign_path = []
     try:
-        index = self.elts.index(node)
+        index = self.elts.index(node)  # type: ignore[arg-type]
     except ValueError as exc:
         raise InferenceError(
             "Tried to retrieve a node {node!r} which does not exist",
@@ -327,10 +374,10 @@ nodes.List.assigned_stmts = sequence_assigned_stmts
 
 
 def assend_assigned_stmts(
-    self: Union[nodes.AssignName, nodes.AssignAttr],
+    self: nodes.AssignName | nodes.AssignAttr,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     return self.parent.assigned_stmts(node=self, context=context)
 
@@ -339,7 +386,9 @@ nodes.AssignName.assigned_stmts = assend_assigned_stmts
 nodes.AssignAttr.assigned_stmts = assend_assigned_stmts
 
 
-def _arguments_infer_argname(self, name, context):
+def _arguments_infer_argname(
+    self, name: str | None, context: InferenceContext
+) -> Generator[InferenceResult, None, None]:
     # arguments information may be missing, in which case we can't do anything
     # more
     if not (self.arguments or self.vararg or self.kwarg):
@@ -401,27 +450,30 @@ def _arguments_infer_argname(self, name, context):
 def arguments_assigned_stmts(
     self: nodes.Arguments,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
-    if context.callcontext:
+    try:
+        node_name = node.name  # type: ignore[union-attr]
+    except AttributeError:
+        # Added to handle edge cases where node.name is not defined.
+        # https://github.com/PyCQA/astroid/pull/1644#discussion_r901545816
+        node_name = None  # pragma: no cover
+
+    if context and context.callcontext:
         callee = context.callcontext.callee
         while hasattr(callee, "_proxied"):
             callee = callee._proxied
     else:
-        callee = None
-    if (
-        context.callcontext
-        and node
-        and getattr(callee, "name", None) == node.frame(future=True).name
-    ):
+        return _arguments_infer_argname(self, node_name, context)
+    if node and getattr(callee, "name", None) == node.frame(future=True).name:
         # reset call context/name
         callcontext = context.callcontext
         context = copy_context(context)
         context.callcontext = None
         args = arguments.CallSite(callcontext, context=context)
-        return args.infer_argument(self.parent, node.name, context)
-    return _arguments_infer_argname(self, node.name, context)
+        return args.infer_argument(self.parent, node_name, context)
+    return _arguments_infer_argname(self, node_name, context)
 
 
 nodes.Arguments.assigned_stmts = arguments_assigned_stmts
@@ -429,10 +481,10 @@ nodes.Arguments.assigned_stmts = arguments_assigned_stmts
 
 @decorators.raise_if_nothing_inferred
 def assign_assigned_stmts(
-    self: Union[nodes.AugAssign, nodes.Assign, nodes.AnnAssign],
+    self: nodes.AugAssign | nodes.Assign | nodes.AnnAssign,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     if not assign_path:
         yield self.value
@@ -441,14 +493,19 @@ def assign_assigned_stmts(
         self.value.infer(context), assign_path, context
     )
 
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 def assign_annassigned_stmts(
     self: nodes.AnnAssign,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     for inferred in assign_assigned_stmts(self, node, context, assign_path):
         if inferred is None:
@@ -463,7 +520,7 @@ nodes.AugAssign.assigned_stmts = assign_assigned_stmts
 
 
 def _resolve_assignment_parts(parts, assign_path, context):
-    """recursive function to resolve multiple assignments"""
+    """Recursive function to resolve multiple assignments."""
     assign_path = assign_path[:]
     index = assign_path.pop(0)
     for part in parts:
@@ -506,15 +563,20 @@ def _resolve_assignment_parts(parts, assign_path, context):
 def excepthandler_assigned_stmts(
     self: nodes.ExceptHandler,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     for assigned in node_classes.unpack_infer(self.type):
         if isinstance(assigned, nodes.ClassDef):
             assigned = objects.ExceptionInstance(assigned)
 
         yield assigned
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.ExceptHandler.assigned_stmts = excepthandler_assigned_stmts
@@ -562,8 +624,8 @@ def _infer_context_manager(self, mgr, context):
 def with_assigned_stmts(
     self: nodes.With,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     """Infer names and other nodes from a *with* statement.
 
@@ -623,14 +685,19 @@ def with_assigned_stmts(
                     ) from exc
                 except TypeError as exc:
                     raise InferenceError(
-                        "Tried to unpack a non-iterable value " "in {node!r}.",
+                        "Tried to unpack a non-iterable value in {node!r}.",
                         node=self,
                         targets=node,
                         assign_path=assign_path,
                         context=context,
                     ) from exc
             yield obj
-    return dict(node=self, unknown=node, assign_path=assign_path, context=context)
+    return {
+        "node": self,
+        "unknown": node,
+        "assign_path": assign_path,
+        "context": context,
+    }
 
 
 nodes.With.assigned_stmts = with_assigned_stmts
@@ -640,10 +707,10 @@ nodes.With.assigned_stmts = with_assigned_stmts
 def named_expr_assigned_stmts(
     self: nodes.NamedExpr,
     node: node_classes.AssignedStmtsPossibleNode,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
-    """Infer names and other nodes from an assignment expression"""
+    """Infer names and other nodes from an assignment expression."""
     if self.target == node:
         yield from self.value.infer(context=context)
     else:
@@ -659,11 +726,11 @@ nodes.NamedExpr.assigned_stmts = named_expr_assigned_stmts
 
 
 @decorators.yes_if_nothing_inferred
-def starred_assigned_stmts(
+def starred_assigned_stmts(  # noqa: C901
     self: nodes.Starred,
     node: node_classes.AssignedStmtsPossibleNode = None,
-    context: Optional[InferenceContext] = None,
-    assign_path: Optional[List[int]] = None,
+    context: InferenceContext | None = None,
+    assign_path: list[int] | None = None,
 ) -> Any:
     """
     Arguments:
@@ -674,8 +741,11 @@ def starred_assigned_stmts(
             A list of indices, where each index specifies what item to fetch from
             the inference results.
     """
+
     # pylint: disable=too-many-locals,too-many-statements
-    def _determine_starred_iteration_lookups(starred, target, lookups):
+    def _determine_starred_iteration_lookups(
+        starred: nodes.Starred, target: nodes.Tuple, lookups: list[tuple[int, int]]
+    ) -> None:
         # Determine the lookups for the rhs of the iteration
         itered = target.itered()
         for index, element in enumerate(itered):
@@ -692,7 +762,7 @@ def starred_assigned_stmts(
     stmt = self.statement(future=True)
     if not isinstance(stmt, (nodes.Assign, nodes.For)):
         raise InferenceError(
-            "Statement {stmt!r} enclosing {node!r} " "must be an Assign or For node.",
+            "Statement {stmt!r} enclosing {node!r} must be an Assign or For node.",
             node=self,
             stmt=stmt,
             unknown=node,
@@ -705,10 +775,13 @@ def starred_assigned_stmts(
     if isinstance(stmt, nodes.Assign):
         value = stmt.value
         lhs = stmt.targets[0]
+        if not isinstance(lhs, nodes.BaseContainer):
+            yield util.Uninferable
+            return
 
         if sum(1 for _ in lhs.nodes_of_class(nodes.Starred)) > 1:
             raise InferenceError(
-                "Too many starred arguments in the " " assignment targets {lhs!r}.",
+                "Too many starred arguments in the assignment targets {lhs!r}.",
                 node=self,
                 targets=lhs,
                 unknown=node,
@@ -725,7 +798,7 @@ def starred_assigned_stmts(
             return
 
         try:
-            elts = collections.deque(rhs.itered())
+            elts = collections.deque(rhs.itered())  # type: ignore[union-attr]
         except TypeError:
             yield util.Uninferable
             return
@@ -774,7 +847,7 @@ def starred_assigned_stmts(
             yield util.Uninferable
             return
         try:
-            itered = inferred_iterable.itered()
+            itered = inferred_iterable.itered()  # type: ignore[union-attr]
         except TypeError:
             yield util.Uninferable
             return
@@ -787,7 +860,7 @@ def starred_assigned_stmts(
                 context=context,
             )
 
-        lookups = []
+        lookups: list[tuple[int, int]] = []
         _determine_starred_iteration_lookups(self, target, lookups)
         if not lookups:
             raise InferenceError(
@@ -802,10 +875,9 @@ def starred_assigned_stmts(
             last_element_index,
             None if is_starred_last else (last_element_length - last_element_index),
         )
-        lookups[-1] = lookup_slice
+        last_lookup = lookup_slice
 
         for element in itered:
-
             # We probably want to infer the potential values *for each* element in an
             # iterable, but we can't infer a list of all values, when only a list of
             # step values are expected:
@@ -817,15 +889,17 @@ def starred_assigned_stmts(
             # which astroid can't know about.
 
             found_element = None
-            for lookup in lookups:
+            for index, lookup in enumerate(lookups):
                 if not hasattr(element, "itered"):
                     break
-                if not isinstance(lookup, slice):
+                if index + 1 is len(lookups):
+                    cur_lookup: slice | int = last_lookup
+                else:
                     # Grab just the index, not the whole length
-                    lookup = lookup[0]
+                    cur_lookup = lookup[0]
                 try:
                     itered_inner_element = element.itered()
-                    element = itered_inner_element[lookup]
+                    element = itered_inner_element[cur_lookup]
                 except IndexError:
                     break
                 except TypeError:
@@ -855,8 +929,8 @@ nodes.Starred.assigned_stmts = starred_assigned_stmts
 def match_mapping_assigned_stmts(
     self: nodes.MatchMapping,
     node: nodes.AssignName,
-    context: Optional[InferenceContext] = None,
-    assign_path: Literal[None] = None,
+    context: InferenceContext | None = None,
+    assign_path: None = None,
 ) -> Generator[nodes.NodeNG, None, None]:
     """Return empty generator (return -> raises StopIteration) so inferred value
     is Uninferable.
@@ -872,8 +946,8 @@ nodes.MatchMapping.assigned_stmts = match_mapping_assigned_stmts
 def match_star_assigned_stmts(
     self: nodes.MatchStar,
     node: nodes.AssignName,
-    context: Optional[InferenceContext] = None,
-    assign_path: Literal[None] = None,
+    context: InferenceContext | None = None,
+    assign_path: None = None,
 ) -> Generator[nodes.NodeNG, None, None]:
     """Return empty generator (return -> raises StopIteration) so inferred value
     is Uninferable.
@@ -889,8 +963,8 @@ nodes.MatchStar.assigned_stmts = match_star_assigned_stmts
 def match_as_assigned_stmts(
     self: nodes.MatchAs,
     node: nodes.AssignName,
-    context: Optional[InferenceContext] = None,
-    assign_path: Literal[None] = None,
+    context: InferenceContext | None = None,
+    assign_path: None = None,
 ) -> Generator[nodes.NodeNG, None, None]:
     """Infer MatchAs as the Match subject if it's the only MatchCase pattern
     else raise StopIteration to yield Uninferable.
