@@ -10,11 +10,14 @@ from __future__ import annotations
 
 import builtins
 import inspect
+import io
+import logging
 import os
 import sys
 import types
 import warnings
 from collections.abc import Iterable
+from contextlib import redirect_stderr, redirect_stdout
 from typing import Any, Union
 
 from astroid import bases, nodes
@@ -22,6 +25,9 @@ from astroid.const import _EMPTY_OBJECT_MARKER, IS_PYPY
 from astroid.manager import AstroidManager
 from astroid.nodes import node_classes
 
+logger = logging.getLogger(__name__)
+
+
 _FunctionTypes = Union[
     types.FunctionType,
     types.MethodType,
@@ -471,7 +477,26 @@ class InspectBuilder:
             # check if it sounds valid and then add an import node, else use a
             # dummy node
             try:
-                getattr(sys.modules[modname], name)
+                with redirect_stderr(io.StringIO()) as stderr, redirect_stdout(
+                    io.StringIO()
+                ) as stdout:
+                    getattr(sys.modules[modname], name)
+                    stderr_value = stderr.getvalue()
+                    if stderr_value:
+                        logger.error(
+                            "Captured stderr while getting %s from %s:\n%s",
+                            name,
+                            sys.modules[modname],
+                            stderr_value,
+                        )
+                    stdout_value = stdout.getvalue()
+                    if stdout_value:
+                        logger.info(
+                            "Captured stdout while getting %s from %s:\n%s",
+                            name,
+                            sys.modules[modname],
+                            stdout_value,
+                        )
             except (KeyError, AttributeError):
                 attach_dummy_node(node, name, member)
             else:
@@ -550,6 +575,24 @@ def _astroid_bootstrapping() -> None:
         )
         bases.AsyncGenerator._proxied = _AsyncGeneratorType
         builder.object_build(bases.AsyncGenerator._proxied, types.AsyncGeneratorType)
+
+    if hasattr(types, "UnionType"):
+        _UnionTypeType = nodes.ClassDef(types.UnionType.__name__)
+        _UnionTypeType.parent = astroid_builtin
+        union_type_doc_node = (
+            nodes.Const(value=types.UnionType.__doc__)
+            if types.UnionType.__doc__
+            else None
+        )
+        _UnionTypeType.postinit(
+            bases=[],
+            body=[],
+            decorators=None,
+            doc_node=union_type_doc_node,
+        )
+        bases.UnionType._proxied = _UnionTypeType
+        builder.object_build(bases.UnionType._proxied, types.UnionType)
+
     builtin_types = (
         types.GetSetDescriptorType,
         types.GeneratorType,
