@@ -6,9 +6,14 @@ from pathlib import Path
 
 from contributors_txt import create_contributors_txt
 
-ASTROID_BASE_DIRECTORY = Path(__file__).parent.parent
-ALIASES_FILE = ASTROID_BASE_DIRECTORY / "script/.contributors_aliases.json"
-DEFAULT_CONTRIBUTOR_PATH = ASTROID_BASE_DIRECTORY / "CONTRIBUTORS.txt"
+CWD = Path(".").absolute()
+ASTROID_BASE_DIRECTORY = Path(__file__).parent.parent.absolute()
+ALIASES_FILE = (
+    ASTROID_BASE_DIRECTORY / "script/.contributors_aliases.json"
+).relative_to(CWD)
+DEFAULT_CONTRIBUTOR_PATH = (ASTROID_BASE_DIRECTORY / "CONTRIBUTORS.txt").relative_to(
+    CWD
+)
 
 
 def main():
