@@ -74,7 +74,8 @@ def test_inference_non_field_default(module: str):
 @parametrize_module
 def test_inference_field_default(module: str):
     """Test inference of dataclass attribute with a field call default
-    (default keyword argument given)."""
+    (default keyword argument given).
+    """
     klass, instance = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -105,7 +106,8 @@ def test_inference_field_default(module: str):
 @parametrize_module
 def test_inference_field_default_factory(module: str):
     """Test inference of dataclass attribute with a field call default
-    (default_factory keyword argument given)."""
+    (default_factory keyword argument given).
+    """
     klass, instance = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -374,6 +376,67 @@ def test_inference_inherited(module: str):
     assert inferred[1].name == "str"
 
 
+def test_dataclass_order_of_inherited_attributes():
+    """Test that an attribute in a child does not get put at the end of the init."""
+    child, normal, keyword_only = astroid.extract_node(
+        """
+    from dataclass import dataclass
+
+
+    @dataclass
+    class Parent:
+        a: str
+        b: str
+
+
+    @dataclass
+    class Child(Parent):
+        c: str
+        a: str
+
+
+    @dataclass(kw_only=True)
+    class KeywordOnlyParent:
+        a: int
+        b: str
+
+
+    @dataclass
+    class NormalChild(KeywordOnlyParent):
+        c: str
+        a: str
+
+
+    @dataclass(kw_only=True)
+    class KeywordOnlyChild(KeywordOnlyParent):
+        c: str
+        a: str
+
+
+    Child.__init__  #@
+    NormalChild.__init__  #@
+    KeywordOnlyChild.__init__  #@
+    """
+    )
+    child_init: bases.UnboundMethod = next(child.infer())
+    assert [a.name for a in child_init.args.args] == ["self", "a", "b", "c"]
+
+    normal_init: bases.UnboundMethod = next(normal.infer())
+    if PY310_PLUS:
+        assert [a.name for a in normal_init.args.args] == ["self", "a", "c"]
+        assert [a.name for a in normal_init.args.kwonlyargs] == ["b"]
+    else:
+        assert [a.name for a in normal_init.args.args] == ["self", "a", "b", "c"]
+        assert [a.name for a in normal_init.args.kwonlyargs] == []
+
+    keyword_only_init: bases.UnboundMethod = next(keyword_only.infer())
+    if PY310_PLUS:
+        assert [a.name for a in keyword_only_init.args.args] == ["self"]
+        assert [a.name for a in keyword_only_init.args.kwonlyargs] == ["a", "b", "c"]
+    else:
+        assert [a.name for a in keyword_only_init.args.args] == ["self", "a", "b", "c"]
+
+
 def test_pydantic_field() -> None:
     """Test that pydantic.Field attributes are currently Uninferable.
 
@@ -406,7 +469,7 @@ def test_pydantic_field() -> None:
 
 @parametrize_module
 def test_init_empty(module: str):
-    """Test init for a dataclass with no attributes"""
+    """Test init for a dataclass with no attributes."""
     node = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -424,7 +487,7 @@ def test_init_empty(module: str):
 
 @parametrize_module
 def test_init_no_defaults(module: str):
-    """Test init for a dataclass with attributes and no defaults"""
+    """Test init for a dataclass with attributes and no defaults."""
     node = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -451,7 +514,7 @@ def test_init_no_defaults(module: str):
 
 @parametrize_module
 def test_init_defaults(module: str):
-    """Test init for a dataclass with attributes and some defaults"""
+    """Test init for a dataclass with attributes and some defaults."""
     node = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -486,7 +549,7 @@ def test_init_defaults(module: str):
 
 @parametrize_module
 def test_init_initvar(module: str):
-    """Test init for a dataclass with attributes and an InitVar"""
+    """Test init for a dataclass with attributes and an InitVar."""
     node = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -602,7 +665,8 @@ def test_init_override(module: str):
 
 @parametrize_module
 def test_init_attributes_from_superclasses(module: str):
-    """Test init for a dataclass that inherits and overrides attributes from superclasses.
+    """Test init for a dataclass that inherits and overrides attributes from
+    superclasses.
 
     Based on https://github.com/PyCQA/pylint/issues/3201
     """
@@ -625,18 +689,20 @@ def test_init_attributes_from_superclasses(module: str):
     """
     )
     init = next(node.infer())
-    assert [a.name for a in init.args.args] == ["self", "arg0", "arg1", "arg2"]
+    assert [a.name for a in init.args.args] == ["self", "arg0", "arg2", "arg1"]
     assert [a.as_string() if a else None for a in init.args.annotations] == [
         None,
         "float",
-        "int",
         "list",  # not str
+        "int",
     ]
 
 
 @parametrize_module
 def test_invalid_init(module: str):
-    """Test that astroid doesn't generate an initializer when attribute order is invalid."""
+    """Test that astroid doesn't generate an initializer when attribute order is
+    invalid.
+    """
     node = astroid.extract_node(
         f"""
     from {module} import dataclass
@@ -655,7 +721,9 @@ def test_invalid_init(module: str):
 
 @parametrize_module
 def test_annotated_enclosed_field_call(module: str):
-    """Test inference of dataclass attribute with a field call in another function call"""
+    """Test inference of dataclass attribute with a field call in another function
+    call.
+    """
     node = astroid.extract_node(
         f"""
     from {module} import dataclass, field
@@ -826,6 +894,49 @@ def test_kw_only_decorator() -> None:
         assert [a.name for a in dee_init.args.kwonlyargs] == []
 
 
+def test_kw_only_in_field_call() -> None:
+    """Test that keyword only fields get correctly put at the end of the __init__."""
+
+    first, second, third = astroid.extract_node(
+        """
+    from dataclasses import dataclass, field
+
+    @dataclass
+    class Parent:
+        p1: int = field(kw_only=True, default=0)
+
+    @dataclass
+    class Child(Parent):
+        c1: str
+
+    @dataclass(kw_only=True)
+    class GrandChild(Child):
+        p2: int = field(kw_only=False, default=1)
+        p3: int = field(kw_only=True, default=2)
+
+    Parent.__init__  #@
+    Child.__init__ #@
+    GrandChild.__init__ #@
+    """
+    )
+
+    first_init: bases.UnboundMethod = next(first.infer())
+    assert [a.name for a in first_init.args.args] == ["self"]
+    assert [a.name for a in first_init.args.kwonlyargs] == ["p1"]
+    assert [d.value for d in first_init.args.kw_defaults] == [0]
+
+    second_init: bases.UnboundMethod = next(second.infer())
+    assert [a.name for a in second_init.args.args] == ["self", "c1"]
+    assert [a.name for a in second_init.args.kwonlyargs] == ["p1"]
+    assert [d.value for d in second_init.args.kw_defaults] == [0]
+
+    third_init: bases.UnboundMethod = next(third.infer())
+    assert [a.name for a in third_init.args.args] == ["self", "c1", "p2"]
+    assert [a.name for a in third_init.args.kwonlyargs] == ["p1", "p3"]
+    assert [d.value for d in third_init.args.defaults] == [1]
+    assert [d.value for d in third_init.args.kw_defaults] == [0, 2]
+
+
 def test_dataclass_with_unknown_base() -> None:
     """Regression test for dataclasses with unknown base classes.
 
@@ -985,8 +1096,8 @@ def test_dataclass_with_multiple_inheritance() -> None:
     assert [a.value for a in overwritten_init.args.defaults] == ["2"]
 
     overwriting_init: bases.UnboundMethod = next(overwriting.infer())
-    assert [a.name for a in overwriting_init.args.args] == ["self", "_abc", "ef"]
-    assert [a.value for a in overwriting_init.args.defaults] == [1.0, 2.0]
+    assert [a.name for a in overwriting_init.args.args] == ["self", "ef", "_abc"]
+    assert [a.value for a in overwriting_init.args.defaults] == [2.0, 1.0]
 
     mixed_init: bases.UnboundMethod = next(mixed.infer())
     assert [a.name for a in mixed_init.args.args] == ["self", "_abc", "ghi"]
@@ -1062,6 +1173,62 @@ def test_dataclass_non_default_argument_after_default() -> None:
     assert next(impossible.infer()) is Uninferable
 
 
+def test_dataclass_with_field_init_is_false() -> None:
+    """When init=False it shouldn't end up in the __init__."""
+    first, second, second_child, third_child, third = astroid.extract_node(
+        """
+    from dataclasses import dataclass, field
+
+
+    @dataclass
+    class First:
+        a: int
+
+    @dataclass
+    class Second(First):
+        a: int = field(init=False, default=1)
+
+    @dataclass
+    class SecondChild(Second):
+        a: float
+
+    @dataclass
+    class ThirdChild(SecondChild):
+        a: str
+
+    @dataclass
+    class Third(First):
+        a: str
+
+    First.__init__  #@
+    Second.__init__  #@
+    SecondChild.__init__  #@
+    ThirdChild.__init__  #@
+    Third.__init__  #@
+    """
+    )
+
+    first_init: bases.UnboundMethod = next(first.infer())
+    assert [a.name for a in first_init.args.args] == ["self", "a"]
+    assert [a.value for a in first_init.args.defaults] == []
+
+    second_init: bases.UnboundMethod = next(second.infer())
+    assert [a.name for a in second_init.args.args] == ["self"]
+    assert [a.value for a in second_init.args.defaults] == []
+
+    second_child_init: bases.UnboundMethod = next(second_child.infer())
+    assert [a.name for a in second_child_init.args.args] == ["self", "a"]
+    assert [a.value for a in second_child_init.args.defaults] == [1]
+
+    third_child_init: bases.UnboundMethod = next(third_child.infer())
+    assert [a.name for a in third_child_init.args.args] == ["self", "a"]
+    assert [a.value for a in third_child_init.args.defaults] == [1]
+
+    third_init: bases.UnboundMethod = next(third.infer())
+    assert [a.name for a in third_init.args.args] == ["self", "a"]
+    assert [a.value for a in third_init.args.defaults] == []
+
+
 def test_dataclass_inits_of_non_dataclasses() -> None:
     """Regression test for __init__ mangling for non dataclasses.
 
@@ -1114,3 +1281,72 @@ def test_dataclass_inits_of_non_dataclasses() -> None:
     third_init: bases.UnboundMethod = next(third.infer())
     assert [a.name for a in third_init.args.args] == ["self", "ef"]
     assert [a.value for a in third_init.args.defaults] == [3]
+
+
+def test_dataclass_with_properties() -> None:
+    """Tests for __init__ creation for dataclasses that use properties."""
+    first, second, third = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+
+    @dataclass
+    class Dataclass:
+        attr: int
+
+        @property
+        def attr(self) -> int:
+            return 1
+
+        @attr.setter
+        def attr(self, value: int) -> None:
+            pass
+
+    class ParentOne(Dataclass):
+        '''Docstring'''
+
+    @dataclass
+    class ParentTwo(Dataclass):
+        '''Docstring'''
+
+    Dataclass.__init__  #@
+    ParentOne.__init__  #@
+    ParentTwo.__init__  #@
+    """
+    )
+
+    first_init: bases.UnboundMethod = next(first.infer())
+    assert [a.name for a in first_init.args.args] == ["self", "attr"]
+    assert [a.value for a in first_init.args.defaults] == [1]
+
+    second_init: bases.UnboundMethod = next(second.infer())
+    assert [a.name for a in second_init.args.args] == ["self", "attr"]
+    assert [a.value for a in second_init.args.defaults] == [1]
+
+    third_init: bases.UnboundMethod = next(third.infer())
+    assert [a.name for a in third_init.args.args] == ["self", "attr"]
+    assert [a.value for a in third_init.args.defaults] == [1]
+
+    fourth = astroid.extract_node(
+        """
+    from dataclasses import dataclass
+
+    @dataclass
+    class Dataclass:
+        other_attr: str
+        attr: str
+
+        @property
+        def attr(self) -> str:
+            return self.other_attr[-1]
+
+        @attr.setter
+        def attr(self, value: int) -> None:
+            pass
+
+    Dataclass.__init__  #@
+    """
+    )
+
+    fourth_init: bases.UnboundMethod = next(fourth.infer())
+    assert [a.name for a in fourth_init.args.args] == ["self", "other_attr", "attr"]
+    assert [a.name for a in fourth_init.args.defaults] == ["Uninferable"]
