@@ -2,14 +2,14 @@
 # For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 # Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
 
-"""Tests for function call inference"""
+"""Tests for function call inference."""
 
 from astroid import bases, builder, nodes
 from astroid.util import Uninferable
 
 
 def test_no_return() -> None:
-    """Test function with no return statements"""
+    """Test function with no return statements."""
     node = builder.extract_node(
         """
     def f():
@@ -25,7 +25,7 @@ def test_no_return() -> None:
 
 
 def test_one_return() -> None:
-    """Test function with a single return that always executes"""
+    """Test function with a single return that always executes."""
     node = builder.extract_node(
         """
     def f():
@@ -42,7 +42,7 @@ def test_one_return() -> None:
 
 
 def test_one_return_possible() -> None:
-    """Test function with a single return that only sometimes executes
+    """Test function with a single return that only sometimes executes.
 
     Note: currently, inference doesn't handle this type of control flow
     """
@@ -63,7 +63,7 @@ def test_one_return_possible() -> None:
 
 
 def test_multiple_returns() -> None:
-    """Test function with multiple returns"""
+    """Test function with multiple returns."""
     node = builder.extract_node(
         """
     def f(x):
@@ -85,7 +85,7 @@ def test_multiple_returns() -> None:
 
 
 def test_argument() -> None:
-    """Test function whose return value uses its arguments"""
+    """Test function whose return value uses its arguments."""
     node = builder.extract_node(
         """
     def f(x, y):
@@ -102,7 +102,7 @@ def test_argument() -> None:
 
 
 def test_inner_call() -> None:
-    """Test function where return value is the result of a separate function call"""
+    """Test function where return value is the result of a separate function call."""
     node = builder.extract_node(
         """
     def f():
