@@ -16,7 +16,9 @@ from astroid.const import IS_PYPY, PY38, PY38_PLUS, PY39_PLUS, PY310_PLUS
     reason="end_lineno and end_col_offset were added in PY38",
 )
 class TestEndLinenoNotSet:
-    """Test 'end_lineno' and 'end_col_offset' are initialized as 'None' for Python < 3.8."""
+    """Test 'end_lineno' and 'end_col_offset' are initialized as 'None' for Python <
+    3.8.
+    """
 
     @staticmethod
     def test_end_lineno_not_set() -> None:
@@ -45,7 +47,9 @@ class TestEndLinenoNotSet:
     reason="end_lineno and end_col_offset were added in PY38",
 )
 class TestLinenoColOffset:
-    """Test 'lineno', 'col_offset', 'end_lineno', and 'end_col_offset' for all nodes."""
+    """Test 'lineno', 'col_offset', 'end_lineno', and 'end_col_offset' for all
+    nodes.
+    """
 
     @staticmethod
     def test_end_lineno_container() -> None:
@@ -661,9 +665,10 @@ class TestLinenoColOffset:
         assert isinstance(f1.args.kwonlyargs[0], nodes.AssignName)
         assert (f1.args.kwonlyargs[0].lineno, f1.args.kwonlyargs[0].col_offset) == (4, 4)
         assert (f1.args.kwonlyargs[0].end_lineno, f1.args.kwonlyargs[0].end_col_offset) == (4, 16)
-        assert isinstance(f1.args.kwonlyargs_annotations[0], nodes.Name)
-        assert (f1.args.kwonlyargs_annotations[0].lineno, f1.args.kwonlyargs_annotations[0].col_offset) == (4, 13)
-        assert (f1.args.kwonlyargs_annotations[0].end_lineno, f1.args.kwonlyargs_annotations[0].end_col_offset) == (4, 16)
+        annotations = f1.args.kwonlyargs_annotations
+        assert isinstance(annotations[0], nodes.Name)
+        assert (annotations[0].lineno, annotations[0].col_offset) == (4, 13)
+        assert (annotations[0].end_lineno, annotations[0].end_col_offset) == (4, 16)
         assert isinstance(f1.args.kw_defaults[0], nodes.Const)
         assert (f1.args.kw_defaults[0].lineno, f1.args.kw_defaults[0].col_offset) == (4, 19)
         assert (f1.args.kw_defaults[0].end_lineno, f1.args.kw_defaults[0].end_col_offset) == (4, 20)
@@ -1231,7 +1236,7 @@ class TestLinenoColOffset:
 
     @staticmethod
     def test_end_lineno_module() -> None:
-        """Tests for Module"""
+        """Tests for Module."""
         code = """print()"""
         module = astroid.parse(code)
         assert isinstance(module, nodes.Module)
